import { types, hasParent, getParent, destroy } from 'mobx-state-tree';
import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { withStyles } from '@material-ui/core/styles';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import Avatar from '@material-ui/core/Avatar';
import ImageIcon from '@material-ui/icons/Image';
import ReactDOM from 'react-dom';
import ButtonBase from '@material-ui/core/ButtonBase';
import Popover from '@material-ui/core/Popover';
import Typography from '@material-ui/core/Typography';
import { TextField, IconButton as IconButton$1, Button } from '@material-ui/core';
import { observable, action, decorate } from 'mobx';
import { observer } from 'mobx-react';
import EditIcon from '@material-ui/icons/Edit';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import TableCell from '@material-ui/core/TableCell';
import Tooltip from '@material-ui/core/Tooltip';
import Checkbox from '@material-ui/core/Checkbox';
import TableSortLabel from '@material-ui/core/TableSortLabel';
import IconButton from '@material-ui/core/IconButton';
import DeleteIcon from '@material-ui/icons/Delete';
import CopyIcon from '@material-ui/icons/FileCopy';
import Menu from '@material-ui/core/Menu';
import MenuItem from '@material-ui/core/MenuItem';
import MoreVertIcon from '@material-ui/icons/MoreVert';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import FilterListIcon from '@material-ui/icons/FilterList';
import Toolbar from '@material-ui/core/Toolbar';
import { lighten } from '@material-ui/core/styles/colorManipulator';
import Divider from '@material-ui/core/Divider';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import Paper from '@material-ui/core/Paper';
import TablePagination from '@material-ui/core/TablePagination';

var BulkAction = types.model("BulkAction", {
  type: types.enumeration("BulkActionEnum", ["delete", "copy"]),
  tooltip: types.maybeNull(types.string),
  multiple: false
});

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var ButtonAction = types.model("ButtonAction", {
  type: types.enumeration("ButtonActionVariantEnum", ["create", "uplaod"]),
  variant: types.enumeration("ButtonActionVariantEnum", ["text", "flat", "outlined", "contained", "raised"]),
  color: types.enumeration("ButtonActionColorEnum", ["default", "inherit", "primary", "secondary"]),
  size: types.enumeration("ButtonActionSizeEnum", ["small", "medium", "large"])
}).preProcessSnapshot(function (snapshot) {
  var variant = snapshot.variant || "raised";
  var color = snapshot.color || "secondary";
  var size = snapshot.size || "medium";
  return _extends({}, snapshot, { variant: variant, color: color, size: size });
});

var Column = types.model("Column", {
  fieldName: types.string,
  /**
   * The translated display label for this fieldName.
   */
  fieldLabel: types.string,
  /**
   * If true, the View can hide this column
   */
  hideable: types.boolean,
  /**
   * If true, this column is initially hidden
   */
  hidden: types.boolean,
  /**
   *
   */
  align: types.enumeration("ColumnAlignEnum", ["left", "right", "center", "inherit", "justify"]),
  /**
   * If true, the column header can be clicked to sort the content
   */
  sortable: types.boolean,
  columnCountData: types.array(types.frozen())
}).volatile(function (self) {
  return {
    renderer: function renderer(value) {
      return value;
    }
  };
}).actions(function (self) {
  return {
    setRenderer: function setRenderer(func) {
      self.renderer = func;
    },
    toggleColumnVisibility: function toggleColumnVisibility() {
      self.hidden = !self.hidden;
    }
  };
});

var columnBuilder = function columnBuilder(fieldName, fieldLabel, cellRendererFunc) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  var fitToContent = options.hasOwnProperty("fitToContent") ? options.fitToContent : false;
  var hideable = options.hasOwnProperty("hideable") ? options.hideable : true;
  var hidden = options.hasOwnProperty("hidden") ? options.hidden : false;
  var align = options.hasOwnProperty("align") ? options.align : "left";
  var sortable = options.hasOwnProperty("sortable") ? options.sortable : false;
  var columnCountData = options.hasOwnProperty("columnCountData") ? options.columnCountData : undefined;
  var column = Column.create({
    fieldName: fieldName,
    fieldLabel: fieldLabel,
    fitToContent: fitToContent,
    hideable: hideable,
    hidden: hidden,
    align: align,
    sortable: sortable,
    columnCountData: columnCountData
  });
  column.setRenderer(cellRendererFunc);
  return column;
};

var ColumnList = types.model("ColumnList", {
  columns: types.array(Column),
  showCheckbox: true
}).views(function (self) {
  return {
    /**
     * Returns all columns
     */
    get allColumns() {
      return self.columns;
    },
    get numAllColumns() {
      return self.columns.length;
    },
    /**
     * Returns only visible columns
     */
    get visibleColumns() {
      return self.columns.filter(function (x) {
        return !x.hidden;
      });
    },
    get numVisibleColumns() {
      return self.visibleColumns.length;
    },
    /**
     * Returns only columns that can be hidden
     */
    get hideableColumns() {
      return self.columns.filter(function (x) {
        return x.hideable;
      });
    },
    get numHideableColumns() {
      return self.hideableColumns.length;
    },
    get hasHideableColumns() {
      return self.numAllColumns > 0;
    }
  };
});

var prevSelectedDataId = void 0;

var Data = types.model("Data", {
  id: types.identifier,
  fieldNames: types.map(types.frozen()),
  isSelected: false
}).actions(function (self) {
  return {
    updateIsSelected: function updateIsSelected(value, shiftKey) {
      self.isSelected = value;

      if (value && hasParent(self, 2) && shiftKey) {
        getParent(self, 2).shiftSelect(self, prevSelectedDataId);
      }

      if (self.isSelected) {
        prevSelectedDataId = self.id;
      } else {
        prevSelectedDataId = null;
      }
    },

    /**
     * Shortcut for setting a value on fieldNames.
     */
    setFieldValue: function setFieldValue(key, value) {
      self.fieldNames.set(key, value);
    },
    getFieldValue: function getFieldValue(key) {
      return self.fieldNames.get(key);
    }
  };
});

var createFilterable = function createFilterable() {
  return types.model({}).actions(function (self) {
    return {
      setRules: function setRules(func) {
        self.rules = func;
      },
      update: function update(value) {
        self.value = value;
      }
    };
  });
};

var StringFilter = types.compose(types.model("StringFilter", {
  id: types.string,
  value: types.string
}), createFilterable());

var NumberFilter = types.compose(types.model("NumberFilter", {
  id: types.string,
  value: types.number
}), createFilterable());

var BooleanFilter = types.compose(types.model("BooleanFilter", {
  id: types.string,
  value: types.boolean
}), createFilterable());

var ArrayFilter = types.compose(types.model("ArrayFilter", {
  id: types.string,
  value: types.array(types.string)
}), createFilterable());

var Filter = types.union(StringFilter, NumberFilter, BooleanFilter, ArrayFilter);

var RowAction = types.model("RowAction", {
  type: types.enumeration("RowActionEnum", ["delete", "goto", "edit", "copy"]),
  tooltip: types.maybeNull(types.string),
  rules: types.optional(types.map(types.frozen), {})
});

var getSorting = function getSorting(order, orderBy) {
  return order === "desc" ? function (a, b) {
    return b.fieldNames.get(orderBy) < a.fieldNames.get(orderBy) ? -1 : 1;
  } : function (a, b) {
    return a.fieldNames.get(orderBy) < b.fieldNames.get(orderBy) ? -1 : 1;
  };
};

var TableModel = types.model("TableModel", {
  columnList: ColumnList,
  dataProvider: types.array(Data),
  bulkActions: types.array(BulkAction),
  buttonActions: types.array(ButtonAction),
  rowActions: types.array(RowAction),
  orderBy: "",
  order: types.enumeration("orderEnum", ["asc", "desc"]),
  title: "Title",
  selectedTitle: "Selected Title",
  page: 0,
  rowsPerPageOptions: types.array(types.number),
  rowsPerPage: types.number,
  filters: types.array(Filter)
}).preProcessSnapshot(function (snapshot) {
  var order = snapshot.order || "asc";
  var rowsPerPageOptions = snapshot.rowsPerPageOptions || [10, 20, 30, 40, 50];
  var rowsPerPage = snapshot.rowsPerPage || 10;
  return _extends({}, snapshot, { order: order, rowsPerPage: rowsPerPage, rowsPerPageOptions: rowsPerPageOptions });
}).actions(function (self) {
  return {
    shiftSelect: function shiftSelect(caller, prevSelectedDataId) {
      if (prevSelectedDataId) {
        var items = self.filtered;
        var indexA = items.findIndex(function (data) {
          return data.id === prevSelectedDataId;
        });
        var indexB = items.findIndex(function (data) {
          return data.id === caller.id;
        });
        var low = Math.min(indexA, indexB);
        var high = Math.max(indexA, indexB);
        var candidates = items.filter(function (item, index) {
          return index >= low && index <= high;
        });
        candidates.forEach(function (item) {
          return item.updateIsSelected(true);
        });
      }
    },
    updateDataProvider: function updateDataProvider(value) {
      self.dataProvider = value;
    },
    updatePage: function updatePage(page) {
      self.page = page;
    },
    updateRowsPerPage: function updateRowsPerPage(rowsPerPage) {
      self.rowsPerPage = rowsPerPage;
    },
    add: function add(data) {
      self.dataProvider.push(data);
    },
    destroyItems: function destroyItems(items) {
      items.forEach(function (item) {
        destroy(item);
      });
    },
    updateOrderBy: function updateOrderBy(value) {
      if (self.orderBy === value) {
        self.updateOrder(self.order === "asc" ? "desc" : "asc");
      } else {
        self.orderBy = value;
      }
    },
    updateOrder: function updateOrder(value) {
      self.order = value;
    },
    selectAll: function selectAll() {
      self.filtered.forEach(function (data) {
        return data.isSelected = true;
      });
    },
    deselectAll: function deselectAll() {
      self.filtered.forEach(function (data) {
        return data.isSelected = false;
      });
    },

    /**
     * A filter is just a function that return true or false
     * based on conditions that plays well with the Data fields.
     *
     * @param {*} filter
     */
    addFilter: function addFilter(filter) {
      self.filters.push(filter);
    },
    getFilter: function getFilter(id) {
      var filter = self.filters.find(function (filter) {
        return filter.id === id;
      });
      return filter;
    },
    removeFilter: function removeFilter(id) {
      var filter = self.getFilter(id);
      self.filters.remove(filter);
    }
  };
}).views(function (self) {
  return {
    get filtered() {
      var candidates = [].concat(toConsumableArray(self.dataProvider));
      candidates = candidates.filter(function (data) {
        var result = true;
        self.filters.forEach(function (filter) {
          // console.log(filter.id);
          if (filter.rules(data, filter.value) === false) {
            result = false;
          }
        });
        return result;
      });
      return candidates;
    },
    get sorted() {
      return self.filtered.sort(getSorting(self.order, self.orderBy)).slice(self.page * self.rowsPerPage, self.page * self.rowsPerPage + self.rowsPerPage);
    },
    get selected() {
      return self.filtered.filter(function (data) {
        return data.isSelected;
      });
    },
    get numSelected() {
      return self.selected.length;
    },
    get numRowCount() {
      return self.filtered.length;
    },
    get hasSelected() {
      return self.selected.length > 0;
    },
    get hasRowActions() {
      return self.rowActions.length > 0;
    },
    get hasBulkActions() {
      return self.bulkActions.length > 0;
    },
    get hasButtonActions() {
      return self.buttonActions.length > 0;
    },
    get hasFilter() {
      return self.filtered.length < self.dataProvider.length;
    }
  };
});

var styles = function styles(theme) {
  return {
    success: {
      backgroundColor: "#BCF0D2",
      color: "#2AB764",
      borderRadius: 12,
      padding: "4px 12px"
    },
    warn: {
      backgroundColor: "#FEF9E7",
      color: "#C19C0B",
      borderRadius: 12,
      padding: "4px 12px"
    },
    error: {
      backgroundColor: "#FBDEDB",
      color: "#c0392b",
      borderRadius: 12,
      padding: "4px 12px"
    },
    info: {
      backgroundColor: "#CBE5F6",
      color: "#2980b9",
      borderRadius: 12,
      padding: "4px 12px"
    }
  };
};

var CellStatusRenderer = function (_React$Component) {
  inherits(CellStatusRenderer, _React$Component);

  function CellStatusRenderer() {
    classCallCheck(this, CellStatusRenderer);
    return possibleConstructorReturn(this, (CellStatusRenderer.__proto__ || Object.getPrototypeOf(CellStatusRenderer)).apply(this, arguments));
  }

  createClass(CellStatusRenderer, [{
    key: "render",
    value: function render() {
      var _props = this.props,
          classes = _props.classes,
          status = _props.status,
          label = _props.label;


      return React.createElement(
        "div",
        { className: classes[status] },
        label
      );
    }
  }]);
  return CellStatusRenderer;
}(React.Component);

CellStatusRenderer.propTypes = {
  statusList: PropTypes.array.isRequired,
  status: PropTypes.string
};
CellStatusRenderer.defaultProps = {
  statusList: ["error", "warn", "info", "success"],
  stauts: "info"
};


var CellStatusRenderer$1 = withStyles(styles)(CellStatusRenderer);

var CellAvatarRenderer = function (_React$Component) {
  inherits(CellAvatarRenderer, _React$Component);

  function CellAvatarRenderer() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, CellAvatarRenderer);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = CellAvatarRenderer.__proto__ || Object.getPrototypeOf(CellAvatarRenderer)).call.apply(_ref, [this].concat(args))), _this), _this.renderAvatar = function () {
      var _this$props = _this.props,
          avatarSrc = _this$props.avatarSrc,
          initials = _this$props.initials;


      if (avatarSrc) {
        return React.createElement(Avatar, { src: avatarSrc });
      } else if (initials) {
        return React.createElement(
          Avatar,
          null,
          initials.toUpperCase()
        );
      } else {
        return React.createElement(
          Avatar,
          null,
          React.createElement(ImageIcon, null)
        );
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(CellAvatarRenderer, [{
    key: "render",
    value: function render() {
      var _props = this.props,
          primaryLabel = _props.primaryLabel,
          secondaryLabel = _props.secondaryLabel;


      return React.createElement(
        ListItem,
        { dense: true, disableGutters: true },
        this.renderAvatar(),
        React.createElement(ListItemText, { primary: primaryLabel, secondary: secondaryLabel })
      );
    }
  }]);
  return CellAvatarRenderer;
}(React.Component);

CellAvatarRenderer.propTypes = {
  avatarSrc: PropTypes.string,
  initials: PropTypes.string,
  primaryLabel: PropTypes.string,
  secondaryLabel: PropTypes.string
};
CellAvatarRenderer.defaultProps = {
  avatarSrc: null,
  initials: null,
  primaryLabel: null,
  secondaryLabel: null
};

var styles$1 = function styles(theme) {
  return {
    paper: _extends({}, theme.mixins.gutters(), {
      paddingTop: theme.spacing.unit * 2,
      paddingBottom: theme.spacing.unit * 2,
      backgroundColor: "#FAFAFA",
      minWidth: 300
    }),
    popover: {
      marginTop: 1
    },
    root: {
      height: "100%",
      flex: 1,
      margin: 0,
      padding: 0,
      display: "flex",
      justifyContent: "space-between",
      cursor: "pointer"
    },
    focusVisible: {},
    icon: {
      fontSize: 20
    }
  };
};

var CellTextEditable = function (_React$Component) {
  inherits(CellTextEditable, _React$Component);

  function CellTextEditable(props) {
    classCallCheck(this, CellTextEditable);

    var _this = possibleConstructorReturn(this, (CellTextEditable.__proto__ || Object.getPrototypeOf(CellTextEditable)).call(this, props));

    _this.isFocused = false;

    _this.isFocusedUpdate = function (value) {
      return _this.isFocused = value;
    };

    _this.currentState = observable({
      open: false,
      currentValue: ""
    });

    _this.handleClose = function (e) {
      _this.currentState.open = false;

      if (e.key && e.key.toLowerCase() === "escape") return;
      if (_this.props.value !== _this.currentState.currentValue) return;

      _this.props.onValueChanged(_this.currentState.currentValue);
    };

    _this.handleButtonClick = action(function () {
      _this.currentState.open = true;
    });
    _this.handleChange = action(function (e) {
      _this.currentState.currentValue = e.currentTarget.value;
    });

    _this.handleKeyDown = function (e) {
      if (e.keyCode === 13) {
        e.preventDefault();
        _this.props.onValueChanged(_this.currentState.currentValue);
        _this.currentState.open = false;
      }
    };

    _this.handleFocus = function (e) {
      _this.currentState.currentValue = _this.props.value;
      e.currentTarget.setSelectionRange(0, -1);
    };

    _this.handleRootFocus = function (e) {
      _this.isFocusedUpdate(true);
    };

    _this.handleRootBlur = function (e) {
      _this.isFocusedUpdate(false);
    };

    _this.handleButtonBaseFocus = function (e) {
      window.onkeypress = function (e) {
        _this.currentState.currentValue = e.key;
        _this.currentState.open = true;
      };
    };

    _this.handleButtonBaseBlur = function (e) {
      window.onkeypress = undefined;
    };

    _this.handleBlur = function (e) {
      e.preventDefault();
    };

    _this.handleMouseOver = function (e) {
      _this.isFocusedUpdate(true);
    };

    _this.handleMouseOut = function (e) {
      _this.isFocusedUpdate(false);
    };

    _this.anchorEl = React.createRef();
    return _this;
  }

  createClass(CellTextEditable, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.anchorEl = ReactDOM.findDOMNode(this).parentNode;
    }
  }, {
    key: "render",
    value: function render() {
      var _props = this.props,
          value = _props.value,
          classes = _props.classes;


      var hasValue = value && value !== "";

      return React.createElement(
        Fragment,
        null,
        React.createElement(
          "div",
          {
            className: classes.root,
            onFocus: this.handleRootFocus,
            onBlur: this.handleRootBlur,
            onMouseOver: this.handleMouseOver,
            onMouseOut: this.handleMouseOut,
            onClick: this.handleButtonClick
          },
          React.createElement(
            ButtonBase,
            {
              onFocus: this.handleButtonBaseFocus,
              onBlur: this.handleButtonBaseBlur,
              focusRipple: false,
              classes: { focusVisible: classes.focusVisible }
            },
            React.createElement(
              Typography,
              { color: hasValue ? "default" : "textSecondary", align: "left", noWrap: true },
              value && value !== "" ? value : "Add a name"
            )
          ),
          React.createElement(
            "div",
            { style: { opacity: this.isFocused ? 1 : 0 } },
            React.createElement(EditIcon, { className: classes.icon })
          )
        ),
        React.createElement(
          Popover,
          {
            className: classes.popover,
            onClose: this.handleClose,
            anchorEl: this.anchorEl,
            open: this.currentState.open,
            elevation: 3,
            anchorOrigin: {
              vertical: "top",
              horizontal: "left"
            },
            transformOrigin: {
              vertical: "top",
              horizontal: "left"
            }
          },
          React.createElement(
            "div",
            { className: classes.paper },
            React.createElement(TextField, {
              autoFocus: true,
              onFocus: this.handleFocus,
              fullWidth: true,
              value: this.currentState.currentValue,
              onChange: this.handleChange,
              onKeyDown: this.handleKeyDown,
              onBlur: this.handleBlur
            })
          )
        )
      );
    }
  }]);
  return CellTextEditable;
}(React.Component);

decorate(CellTextEditable, {
  isFocused: observable,
  isFocusedUpdate: action
});

var CellTextEditable$1 = withStyles(styles$1)(observer(CellTextEditable));

var MstMuiTableHead = function MstMuiTableHead(_ref) {
  var columnList = _ref.columnList,
      numSelected = _ref.numSelected,
      rowCount = _ref.rowCount,
      rowActions = _ref.rowActions,
      onSelectAllClick = _ref.onSelectAllClick,
      onRequestSort = _ref.onRequestSort,
      order = _ref.order,
      orderBy = _ref.orderBy;

  var createSortHandler = function createSortHandler(property) {
    return function (event) {
      onRequestSort(event, property);
    };
  };

  return React.createElement(
    TableHead,
    null,
    React.createElement(
      TableRow,
      null,
      columnList.showCheckbox && React.createElement(
        TableCell,
        { padding: "checkbox" },
        React.createElement(Checkbox, {
          color: "primary",
          indeterminate: numSelected > 0 && numSelected < rowCount,
          checked: rowCount > 0 && numSelected === rowCount,
          onChange: onSelectAllClick
        })
      ),
      columnList.visibleColumns.map(function (column, i) {
        return React.createElement(
          TableCell,
          {
            key: "MstMuiTableHead_" + column.fieldName,
            align: column.align,
            padding: columnList.showCheckbox && i === 0 ? "none" : "default",
            sortDirection: orderBy === column.fieldName ? order : false
          },
          React.createElement(
            Tooltip,
            { title: "Sort", enterDelay: 300 },
            React.createElement(
              TableSortLabel,
              {
                active: orderBy === column.fieldName,
                direction: order,
                onClick: createSortHandler(column.fieldName)
              },
              column.fieldLabel
            )
          )
        );
      }),
      rowActions.length > 0 && React.createElement(TableCell, { align: "right" })
    )
  );
};

var MstMuiTableHead$1 = observer(MstMuiTableHead);

var MstMuiColumn = function (_React$Component) {
  inherits(MstMuiColumn, _React$Component);

  function MstMuiColumn() {
    classCallCheck(this, MstMuiColumn);
    return possibleConstructorReturn(this, (MstMuiColumn.__proto__ || Object.getPrototypeOf(MstMuiColumn)).apply(this, arguments));
  }

  createClass(MstMuiColumn, [{
    key: "render",
    value: function render() {
      var _props = this.props,
          column = _props.column,
          value = _props.value,
          data = _props.data,
          classes = _props.classes,
          other = objectWithoutProperties(_props, ["column", "value", "data", "classes"]);

      return React.createElement(
        TableCell,
        _extends({ align: column.align }, other),
        column.renderer(value, data)
      );
    }
  }]);
  return MstMuiColumn;
}(React.Component);

var MstMuiColumn$1 = observer(MstMuiColumn);

var supportedIcons = {
  edit: EditIcon,
  delete: DeleteIcon
};

var SingleRowAction = function SingleRowAction(_ref) {
  var onRowAction = _ref.onRowAction,
      rowAction = _ref.rowAction,
      data = _ref.data;

  var handleClick = function handleClick(e) {
    onRowAction(data, rowAction);
  };

  var Icon = supportedIcons[rowAction.type];

  return React.createElement(
    IconButton,
    { onClick: handleClick },
    React.createElement(Icon, null)
  );
};

var supportedIcons$1 = {
  edit: EditIcon,
  delete: DeleteIcon,
  copy: CopyIcon
};

var MultipleRowActions = function (_React$Component) {
  inherits(MultipleRowActions, _React$Component);

  function MultipleRowActions() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, MultipleRowActions);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = MultipleRowActions.__proto__ || Object.getPrototypeOf(MultipleRowActions)).call.apply(_ref, [this].concat(args))), _this), _this.anchorEl = null, _this.anchorElUpdate = function (value) {
      return _this.anchorEl = value;
    }, _this.handleClick = function (e) {
      _this.anchorElUpdate(e.currentTarget);
    }, _this.handleClose = function (e) {
      _this.anchorElUpdate(null);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(MultipleRowActions, [{
    key: "render",
    value: function render() {
      var _props = this.props,
          rowActions = _props.rowActions,
          onRowAction = _props.onRowAction,
          data = _props.data;

      var open = Boolean(this.anchorEl);

      return React.createElement(
        "div",
        null,
        React.createElement(
          IconButton,
          {
            "aria-label": "More",
            "aria-owns": open ? "long-menu" : null,
            "aria-haspopup": "true",
            onClick: this.handleClick
          },
          React.createElement(MoreVertIcon, null)
        ),
        React.createElement(
          Menu,
          {
            id: "long-menu",
            anchorEl: this.anchorEl,
            open: open,
            onClose: this.handleClose
          },
          rowActions.map(function (rowAction) {
            var Icon = supportedIcons$1[rowAction.type];
            return React.createElement(
              MenuItem,
              {
                key: rowAction.type,
                onClick: function onClick() {
                  return onRowAction(data, rowAction);
                }
              },
              React.createElement(
                ListItemIcon,
                null,
                React.createElement(Icon, null)
              ),
              React.createElement(ListItemText, {
                inset: true,
                primary: rowAction.label || rowAction.type
              })
            );
          })
        )
      );
    }
  }]);
  return MultipleRowActions;
}(React.Component);

decorate(MultipleRowActions, {
  anchorEl: observable,
  anchorElUpdate: action
});

var MultipleRowActions$1 = observer(MultipleRowActions);

var shiftKey = false;

var MstMuiTableRow = function (_React$Component) {
  inherits(MstMuiTableRow, _React$Component);

  function MstMuiTableRow() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, MstMuiTableRow);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = MstMuiTableRow.__proto__ || Object.getPrototypeOf(MstMuiTableRow)).call.apply(_ref, [this].concat(args))), _this), _this.handleClick = function (event) {
      return shiftKey = event.shiftKey;
    }, _this.handleChange = function (event, checked) {
      return _this.props.data.updateIsSelected(checked, shiftKey);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(MstMuiTableRow, [{
    key: "render",
    value: function render() {
      var _props = this.props,
          data = _props.data,
          columnList = _props.columnList,
          rowActions = _props.rowActions,
          onRowAction = _props.onRowAction;

      return React.createElement(
        TableRow,
        {
          selected: data.isSelected,
          "aria-checked": data.isSelected,
          role: "checkbox",
          tabIndex: -1
        },
        columnList.showCheckbox && React.createElement(
          TableCell,
          { padding: "checkbox" },
          React.createElement(Checkbox, {
            color: "primary",
            checked: data.isSelected,
            onClick: this.handleClick,
            onChange: this.handleChange
          })
        ),
        columnList.visibleColumns.map(function (column, i) {
          return React.createElement(MstMuiColumn$1, {
            key: "MstMuiColumn_" + column.fieldName,
            column: column,
            value: data.fieldNames.get(column.fieldName),
            data: data,
            padding: columnList.showCheckbox && i === 0 ? "none" : "default"
          });
        }),
        rowActions.length > 0 && React.createElement(
          TableCell,
          { align: "right" },
          rowActions.length === 1 && React.createElement(SingleRowAction, { data: data, rowAction: rowActions[0], onRowAction: onRowAction }),
          rowActions.length > 1 && React.createElement(MultipleRowActions$1, { data: data, rowActions: rowActions, onRowAction: onRowAction })
        )
      );
    }
  }]);
  return MstMuiTableRow;
}(React.Component);

var MstMuiTableRow$1 = observer(MstMuiTableRow);

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var classnames = createCommonjsModule(function (module) {
/*!
  Copyright (c) 2017 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/
/* global define */

(function () {

	var hasOwn = {}.hasOwnProperty;

	function classNames () {
		var classes = [];

		for (var i = 0; i < arguments.length; i++) {
			var arg = arguments[i];
			if (!arg) continue;

			var argType = typeof arg;

			if (argType === 'string' || argType === 'number') {
				classes.push(arg);
			} else if (Array.isArray(arg) && arg.length) {
				var inner = classNames.apply(null, arg);
				if (inner) {
					classes.push(inner);
				}
			} else if (argType === 'object') {
				for (var key in arg) {
					if (hasOwn.call(arg, key) && arg[key]) {
						classes.push(key);
					}
				}
			}
		}

		return classes.join(' ');
	}

	if (module.exports) {
		classNames.default = classNames;
		module.exports = classNames;
	} else {
		window.classNames = classNames;
	}
}());
});

var IconMap = {
  delete: React.createElement(DeleteIcon, null)
};

var BulkActionView = function BulkActionView(_ref) {
  var bulkAction = _ref.bulkAction,
      onBulkAction = _ref.onBulkAction;

  return React.createElement(
    IconButton$1,
    { onClick: function onClick() {
        return onBulkAction(bulkAction);
      } },
    IconMap[bulkAction.type]
  );
};

var BulkActionsView = function BulkActionsView(_ref) {
  var bulkActions = _ref.bulkActions,
      onBulkAction = _ref.onBulkAction;

  return bulkActions.map(function (bulkAction) {
    return React.createElement(BulkActionView, {
      onBulkAction: onBulkAction,
      key: "bulkAction" + bulkAction.type,
      bulkAction: bulkAction
    });
  });
};

var ButtonAction$1 = function ButtonAction(_ref) {
  var buttonAction = _ref.buttonAction,
      onButtonAction = _ref.onButtonAction;

  var handleClick = function handleClick(e) {
    return onButtonAction(buttonAction);
  };

  return React.createElement(
    Button,
    {
      color: buttonAction.color,
      variant: buttonAction.variant,
      onClick: handleClick
    },
    buttonAction.type
  );
};

var ButtonActionsView = function ButtonActionsView(_ref) {
  var buttonActions = _ref.buttonActions,
      onButtonAction = _ref.onButtonAction;

  return buttonActions.map(function (buttonAction) {
    return React.createElement(ButtonAction$1, {
      key: "buttonAction" + buttonAction.type,
      buttonAction: buttonAction,
      onButtonAction: onButtonAction
    });
  });
};

var styles$2 = function styles(theme) {
  //console.log(theme);
  return {
    marker: {
      borderRadius: "50%",
      position: "absolute",
      top: 6,
      right: 6,
      width: 8,
      height: 8,
      backgroundColor: theme.palette.secondary.light,
      pointerEvents: "none"
    }
  };
};

var FilterMarkedAsActive = function FilterMarkedAsActive(_ref) {
  var children = _ref.children,
      markerClassName = _ref.markerClassName;
  return React.createElement(
    "div",
    { style: { position: "relative" } },
    React.createElement(
      "div",
      null,
      children
    ),
    React.createElement("div", { className: markerClassName })
  );
};

var PopoverFilter = function (_React$Component) {
  inherits(PopoverFilter, _React$Component);

  function PopoverFilter() {
    var _ref2;

    var _temp, _this, _ret;

    classCallCheck(this, PopoverFilter);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref2 = PopoverFilter.__proto__ || Object.getPrototypeOf(PopoverFilter)).call.apply(_ref2, [this].concat(args))), _this), _this.showFilter = false, _this.showFilterUpdate = function (value) {
      return _this.showFilter = value;
    }, _this.anchorEl = null, _this.handleClick = function (event) {
      _this.anchorEl = event.currentTarget;
      _this.showFilterUpdate(true);
    }, _this.handleClose = function (event) {
      _this.showFilterUpdate(false);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(PopoverFilter, [{
    key: "render",
    value: function render() {
      var _props = this.props,
          hasFilter = _props.hasFilter,
          classes = _props.classes;


      return React.createElement(
        React.Fragment,
        null,
        hasFilter ? React.createElement(
          FilterMarkedAsActive,
          { markerClassName: classes.marker },
          React.createElement(
            Tooltip,
            { title: "Filter list" },
            React.createElement(
              IconButton,
              { "aria-label": "Filter list", onClick: this.handleClick },
              React.createElement(FilterListIcon, null)
            )
          )
        ) : React.createElement(
          Tooltip,
          { title: "Filter list" },
          React.createElement(
            IconButton,
            { "aria-label": "Filter list", onClick: this.handleClick },
            React.createElement(FilterListIcon, null)
          )
        ),
        React.createElement(
          Popover,
          {
            open: this.showFilter,
            anchorEl: this.anchorEl,
            onClose: this.handleClose,
            anchorOrigin: {
              vertical: "top",
              horizontal: "right"
            },
            transformOrigin: {
              vertical: "top",
              horizontal: "right"
            }
          },
          this.props.children
        )
      );
    }
  }]);
  return PopoverFilter;
}(React.Component);

decorate(PopoverFilter, {
  showFilter: observable,
  updateShowFilter: action
});

var PopoverFilter$1 = withStyles(styles$2)(observer(PopoverFilter));

var styles$3 = function styles(theme) {
  return {
    root: {
      paddingRight: theme.spacing.unit,
      backgroundColor: "#F5F5F5"
    },
    highlight: theme.palette.type === "light" ? {
      color: theme.palette.primary.main,
      backgroundColor: lighten(theme.palette.primary.light, 0.85)
    } : {
      color: theme.palette.text.primary,
      backgroundColor: theme.palette.primary.dark
    },
    spacer: {
      flex: "1 1 100%"
    },
    space: {
      width: 10
    },
    actions: {
      color: theme.palette.text.primary,
      display: "flex",
      alignItems: "center"
    },
    title: {
      display: "flex",
      alignItems: "center",
      flex: "1 0 auto",
      marginRight: 48
    }
  };
};

var MstMuiTableToolbar = function MstMuiTableToolbar(_ref) {
  var classes = _ref.classes,
      numSelected = _ref.numSelected,
      bulkActions = _ref.bulkActions,
      buttonActions = _ref.buttonActions,
      onBulkAction = _ref.onBulkAction,
      onButtonAction = _ref.onButtonAction,
      hasFilter = _ref.hasFilter,
      slotForLeftToolbarArea = _ref.slotForLeftToolbarArea,
      slotForFilterContent = _ref.slotForFilterContent;
  return React.createElement(
    React.Fragment,
    null,
    React.createElement(
      Toolbar,
      {
        className: classnames(classes.root, defineProperty({}, classes.highlight, numSelected > 0))
      },
      React.createElement(
        "div",
        { className: classes.title },
        numSelected > 0 ? React.createElement(
          React.Fragment,
          null,
          React.createElement(
            Typography,
            { color: "inherit", variant: "subheading" },
            numSelected,
            " selected"
          ),
          React.createElement("div", { className: classes.space }),
          React.createElement(BulkActionsView, { bulkActions: bulkActions, onBulkAction: onBulkAction })
        ) : slotForLeftToolbarArea
      ),
      React.createElement(
        "div",
        { className: classes.actions },
        numSelected > 0 ? null : React.createElement(
          React.Fragment,
          null,
          React.createElement(ButtonActionsView, { buttonActions: buttonActions, onButtonAction: onButtonAction }),
          React.createElement("div", { style: { width: 12 } }),
          slotForFilterContent && React.createElement(
            PopoverFilter$1,
            { hasFilter: hasFilter },
            slotForFilterContent
          )
        )
      )
    ),
    React.createElement(Divider, null)
  );
};

var MstMuiTableToolbar$1 = withStyles(styles$3)(observer(MstMuiTableToolbar));

var styles$4 = function styles(theme) {
  return {
    root: {
      width: "100%",
      marginTop: theme.spacing.unit * 3,
      overflowX: "auto"
    },
    table: {}
  };
};

var MstMuiTable = function MstMuiTable(_ref) {
  var classes = _ref.classes,
      tableModel = _ref.tableModel,
      onBulkAction = _ref.onBulkAction,
      onButtonAction = _ref.onButtonAction,
      onRowAction = _ref.onRowAction,
      slotForLeftToolbarArea = _ref.slotForLeftToolbarArea,
      slotForFilterContent = _ref.slotForFilterContent;

  var handleRequestSort = function handleRequestSort(event, property) {
    tableModel.updateOrderBy(property);
  };

  var handleSelectAllClick = function handleSelectAllClick(event, checked) {
    if (checked) {
      tableModel.selectAll();
    } else {
      tableModel.deselectAll();
    }
  };

  var handleChangePage = function handleChangePage(event, page) {
    tableModel.updatePage(page);
  };

  var handleChangeRowsPerPage = function handleChangeRowsPerPage(event) {
    tableModel.updateRowsPerPage(event.target.value);
  };

  if (!tableModel.rowsPerPageOptions.includes(tableModel.rowsPerPage)) {
    throw new Error("rowsPerPage value of " + tableModel.rowsPerPage + " does not match one of the values in rowsPerPageOptions " + tableModel.rowsPerPageOptions);
  }

  return React.createElement(
    Paper,
    { className: classes.root },
    React.createElement(MstMuiTableToolbar$1, {
      numSelected: tableModel.numSelected,
      bulkActions: tableModel.bulkActions,
      buttonActions: tableModel.buttonActions,
      hasFilter: tableModel.hasFilter,
      onBulkAction: onBulkAction,
      onButtonAction: onButtonAction,
      slotForLeftToolbarArea: slotForLeftToolbarArea,
      slotForFilterContent: slotForFilterContent
    }),
    React.createElement(
      Table,
      { className: classes.table },
      React.createElement(MstMuiTableHead$1, {
        rowCount: tableModel.numRowCount,
        numSelected: tableModel.numSelected,
        columnList: tableModel.columnList,
        rowActions: tableModel.rowActions,
        onSelectAllClick: handleSelectAllClick,
        onRequestSort: handleRequestSort,
        order: tableModel.order,
        orderBy: tableModel.orderBy
      }),
      React.createElement(
        TableBody,
        null,
        tableModel.sorted.map(function (data) {
          return React.createElement(MstMuiTableRow$1, {
            data: data,
            key: "MstMuiTableRow_" + data.id,
            columnList: tableModel.columnList,
            rowActions: tableModel.rowActions,
            onRowAction: onRowAction
          });
        })
      )
    ),
    React.createElement(TablePagination, {
      component: "div",
      count: tableModel.numRowCount,
      rowsPerPage: tableModel.rowsPerPage,
      rowsPerPageOptions: tableModel.rowsPerPageOptions,
      page: tableModel.page,
      backIconButtonProps: {
        "aria-label": "Previous Page"
      },
      nextIconButtonProps: {
        "aria-label": "Next Page"
      },
      onChangePage: handleChangePage,
      onChangeRowsPerPage: handleChangeRowsPerPage
    })
  );
};

var MstMuiTable$1 = withStyles(styles$4)(observer(MstMuiTable));

export { BulkAction, ButtonAction, Column, ColumnList, MstMuiTable$1 as MstMuiTable, Data, Filter, RowAction, TableModel, CellStatusRenderer$1 as CellStatusRenderer, CellAvatarRenderer, CellTextEditable$1 as CellTextEditable, columnBuilder };
//# sourceMappingURL=index.es.js.map
